@@ -1,6 +1,6 @@
 /* -*- Mode: c++ -*-
  *
- *  $Id: otcl.c,v 1.22 2004/02/11 01:12:14 heidemann Exp $
+ *  $Id: otcl.c,v 1.25 2011/09/18 20:58:40 tom_henderson Exp $
  *  
  *  Copyright 1993 Massachusetts Institute of Technology
  * 
@@ -55,11 +55,32 @@
 #define ObjVarTablePtr(OBJ) (&(OBJ)->variables.varTable)
 #define compat_Tcl_AddObjErrorInfo(a,b,c) Tcl_AddErrorInfo(a,b)
 #else
+
+#if TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION < 5
 #define ObjVarTablePtr(OBJ) ((OBJ)->variables.varTablePtr)
+#else
+#define TCL_VERSION_8_5_OR_HIGHER
+#define ObjVarTablePtr(OBJ) (&(OBJ)->variables.varTablePtr->table)
+#endif
+
 #define compat_Tcl_AddObjErrorInfo(a,b,c) Tcl_AddObjErrorInfo(a,b,c)
 #endif
 
-
+#ifdef TCL_VERSION_8_5_OR_HIGHER
+#define Tcl_VarHashInitialize(varTablePtr)  varTablePtr = NULL;
+#define Tcl_IsVarHashInitialized(varTablePtr)  varTablePtr == NULL
+#define Tcl_VarHashGetKey(table, hPtr) TclGetString(hPtr->key.objPtr)
+#define Tcl_VarHashGetValue(hPtr) ((Var *) ((char *)hPtr - TclOffset(VarInHash, entry)))
+#define Tcl_CmdInfoGetProc(co) (Proc *)(co->isNativeObjectProc) ? co->objClientData : co->clientData;
+#else
+#define Tcl_VarHashInitialize(varTablePtr) \
+	varTablePtr = (Tcl_HashTable *)ckalloc(sizeof(Tcl_HashTable)); \
+	if (varTablePtr != NULL) Tcl_InitHashTable(varTablePtr, TCL_STRING_KEYS);
+#define Tcl_IsVarHashInitialized(varTablePtr)  varTablePtr != NULL
+#define Tcl_VarHashGetKey(table, hPtr) Tcl_GetHashKey(table, hPtr)
+#define Tcl_VarHashGetValue(hPtr) (Var*)Tcl_GetHashValue(hp);
+#define Tcl_CmdInfoGetProc(co) (Proc*)co->clientData;
+#endif
 
 /*
  * object and class internals
@@ -416,7 +437,7 @@ OTclDispatch(ClientData cd, Tcl_Interp* in, int argc, CONST84 char* argv[]) {
       args = (CONST84 char**)ckalloc((argc+2)*sizeof(char*));
     args[0] = argv[0]; 
     args[1] = argv[0];
-    args[2] = cl ? Tcl_GetCommandName(in, cl->object.id) : "";
+    args[2] = cl ? (char *) Tcl_GetCommandName(in, cl->object.id) : "";
     for (i = 1; i < argc; i++) args[i+2] = argv[i];
 	
     /*
@@ -431,7 +452,7 @@ OTclDispatch(ClientData cd, Tcl_Interp* in, int argc, CONST84 char* argv[]) {
     printf("\n");
     */
 
-    result = (*proc)(cp, in, argc+2, args);
+    result = (*proc)(cp, in, argc+2, (const char **) args);
     /* this adds to the stack trace */
     if (result == TCL_ERROR) {
 	    char msg[150];
@@ -439,7 +460,7 @@ OTclDispatch(ClientData cd, Tcl_Interp* in, int argc, CONST84 char* argv[]) {
 	     * clobbered sometimes => seg fault.
 	     * ---johnh
 	     */
-	    CONST84 char *old_args2 = cl ? Tcl_GetCommandName(in, cl->object.id) : argv[0];
+	    CONST84 char *old_args2 = cl ? (char *) Tcl_GetCommandName(in, cl->object.id) : argv[0];
 	    sprintf(msg, "\n    (%.40s %.40s line %d)",
 		    old_args2, argv[1], in->errorLine);
 	    compat_Tcl_AddObjErrorInfo(in, msg, -1);
@@ -471,10 +492,10 @@ OTclDispatch(ClientData cd, Tcl_Interp* in, int argc, CONST84 char* argv[]) {
       args = (CONST84 char**)ckalloc((argc+3)*sizeof(char*));
     args[0] = argv[0]; 
     args[1] = argv[0];
-    args[2] = cl ? Tcl_GetCommandName(in, cl->object.id) : "";
+    args[2] = cl ? (char *) Tcl_GetCommandName(in, cl->object.id) : "";
     args[3] = "unknown";
     for (i = 1; i < argc; i++) args[i+3] = argv[i];
-    result = (*proc)(cp, in, argc+3, args);
+    result = (*proc)(cp, in, argc+3, (const char **) args);
     if (result == TCL_ERROR) {
 	    char msg[100];
 	    sprintf(msg, "\n    (%.30s unknown line %d)",
@@ -535,9 +556,9 @@ AutoLoader(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
   else if (obj->procs)
     (void) LookupMethod(obj->procs, argv[3], &proc, &cp, 0);
 
-  if (proc && proc != AutoLoader) {
+  if (proc && proc != (Tcl_CmdProc *) AutoLoader) {
     ClientData cdata = (cp != 0) ? cp : (ClientData)obj;
-    return (*proc)(cdata, in, argc, argv);
+    return (*proc)(cdata, in, argc, (const char **) argv);
   }
   
   Tcl_ResetResult(in);
@@ -549,7 +570,7 @@ AutoLoader(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
 
 int
 MakeAuto(Tcl_CmdInfo* proc, CONST84 char* loader) {
-  proc->proc = AutoLoader;
+  proc->proc = (Tcl_CmdProc *) AutoLoader;
   proc->deleteProc = AutoLoaderDP;
   proc->clientData = (ClientData)strcpy(ckalloc(strlen(loader)+1), loader);
   return (proc->clientData != 0);
@@ -563,13 +584,17 @@ MakeAuto(Tcl_CmdInfo* proc, CONST84 char* loader) {
 
 static void
 AddMethod(Tcl_HashTable* methods, CONST84 char* nm, Tcl_CmdProc* pr,
-	  ClientData cd, Tcl_CmdDeleteProc* dp, ClientData dd)
+	  ClientData cd, ClientData ocd, Tcl_CmdDeleteProc* dp, ClientData dd)
 { 
   int nw = 0;
   Tcl_HashEntry *hPtr = Tcl_CreateHashEntry(methods, nm, &nw);
   Tcl_CmdInfo* co = (Tcl_CmdInfo*)ckalloc(sizeof(Tcl_CmdInfo));
   co->proc = pr;
   co->clientData = cd;
+#ifdef TCL_VERSION_8_5_OR_HIGHER
+  co->objClientData = ocd;
+  co->isNativeObjectProc = (ocd != NULL) ? 1 : 0;
+#endif
   co->deleteProc = dp;
   co->deleteData = dd;
   Tcl_SetHashValue(hPtr, (ClientData)co);
@@ -581,7 +606,10 @@ RemoveMethod(Tcl_HashTable* methods, CONST84 char* nm, ClientData cd) {
   Tcl_HashEntry *hPtr = Tcl_FindHashEntry(methods, nm);
   if (hPtr != 0) {
     Tcl_CmdInfo* co = (Tcl_CmdInfo*)Tcl_GetHashValue(hPtr);
+#ifdef TCL_VERSION_8_5_OR_HIGHER
+#else
     if (co->deleteProc != 0) (*co->deleteProc)(co->deleteData);
+#endif
     ckfree((char*)co);
     Tcl_DeleteHashEntry(hPtr);
     return 1;
@@ -701,12 +729,12 @@ MakeProc(Tcl_CmdInfo* proc, Tcl_Interp* in, int argc, CONST84 char* argv[]) {
 }
 
 static void
-ListKeys(Tcl_Interp* in, Tcl_HashTable* table, CONST84 char* pattern) {
+ListKeys(Tcl_Interp* in, Tcl_HashTable* table, CONST84 char* pattern, int isVarHash) {
   Tcl_HashSearch hSrch;
   Tcl_HashEntry* hPtr = table ? Tcl_FirstHashEntry(table, &hSrch) : 0;
   Tcl_ResetResult(in);
   for (; hPtr != 0; hPtr = Tcl_NextHashEntry(&hSrch)) {
-    char* key = Tcl_GetHashKey(table, hPtr);
+    char* key = (isVarHash) ? Tcl_VarHashGetKey(table, hPtr) : Tcl_GetHashKey(table, hPtr);
     if (!pattern || Tcl_StringMatch(key, pattern))
       Tcl_AppendElement(in, key);
   }
@@ -720,7 +748,7 @@ ListInstanceKeys(Tcl_Interp* in, Tcl_HashTable* table, CONST84 char* pattern) {
   Tcl_ResetResult(in);
   for (; hPtr != 0; hPtr = Tcl_NextHashEntry(&hSrch)) {
     OTclObject* obj = (OTclObject*)Tcl_GetHashKey(table, hPtr);
-    CONST84 char* name = Tcl_GetCommandName(in, obj->id);
+    CONST84 char* name = (char *) Tcl_GetCommandName(in, obj->id);
     if (!pattern || Tcl_StringMatch(name, pattern))
       Tcl_AppendElement(in, name);
   }
@@ -741,7 +769,7 @@ ListProcKeys(Tcl_Interp* in, Tcl_HashTable* table, CONST84 char* pattern) {
      * also counts anything to be autoloaded as a proc
      */
 
-    if (proc!=AutoLoader && proc!=ProcInterpId) continue; 
+    if (proc!=(Tcl_CmdProc *) AutoLoader && proc!=ProcInterpId) continue; 
     Tcl_AppendElement(in, key);
   }
 }
@@ -753,7 +781,7 @@ FindProc(Tcl_HashTable* table, CONST84 char* name) {
   if (hPtr) {
     Tcl_CmdInfo* co = (Tcl_CmdInfo*)Tcl_GetHashValue(hPtr);
     if (co->proc == ProcInterpId)
-      return (Proc*)co->clientData;
+      return Tcl_CmdInfoGetProc(co);
   }
   return 0;
 }
@@ -890,7 +918,6 @@ PrimitiveOInit(void* mem, Tcl_Interp* in, CONST84 char* name, OTclClass* cl) {
    * manipulations. looks like one below global 
    */
 
-  Tcl_InitHashTable(ObjVarTablePtr(obj), TCL_STRING_KEYS);
   obj->variables.level = 1;
 #if TCL_MAJOR_VERSION < 8
   obj->variables.argc = 0;
@@ -919,6 +946,10 @@ PrimitiveOInit(void* mem, Tcl_Interp* in, CONST84 char* name, OTclClass* cl) {
   obj->variables.procPtr->numCompiledLocals = 0;
   obj->variables.procPtr->firstLocalPtr = NULL;
   obj->variables.procPtr->lastLocalPtr = NULL;
+#ifdef TCL_VERSION_8_5_OR_HIGHER
+  obj->variables.clientData = NULL;
+  obj->variables.localCachePtr = NULL; 
+#endif
 #endif
 }
 
@@ -959,7 +990,7 @@ PrimitiveODestroyNoFree(ClientData cd) {
      * but under 7.4p1 it is too late, so check with info
      */
 
-    args[0] = Tcl_GetCommandName(in, obj->id);
+    args[0] = (char *) Tcl_GetCommandName(in, obj->id);
     if (Tcl_GetCommandInfo(in, args[0], &info))
       (void) OTclDispatch(cd, in, 2, args);
     obj->id = 0;
@@ -970,22 +1001,25 @@ PrimitiveODestroyNoFree(ClientData cd) {
    * variables unset here were lost from user destroy, and
    * any trace error messages will be swallowed.
    */
+  if (obj->variables.varTablePtr == NULL) goto done;
   hp = Tcl_FirstHashEntry(ObjVarTablePtr(obj), &hs);
   while (hp != 0) {
     for (;;) {
-      Var* vp = (Var*)Tcl_GetHashValue(hp);
+      Var* vp = Tcl_VarHashGetValue(hp);
       if (!TclIsVarUndefined(vp)) break;
       hp = Tcl_NextHashEntry(&hs);
       if (hp == 0)
 	      goto done;
     }
     if (hp != 0) {
-      char* name = Tcl_GetHashKey(ObjVarTablePtr(obj), hp);
+      char* name = Tcl_VarHashGetKey(ObjVarTablePtr(obj), hp);
       (void)OTclUnsetInstVar(obj, in, name, TCL_LEAVE_ERR_MSG);
     }
     hp = Tcl_FirstHashEntry(ObjVarTablePtr(obj), &hs);
   }
 done:
+#ifdef TCL_VERSION_8_5_OR_HIGHER
+#else
   hp = Tcl_FirstHashEntry(ObjVarTablePtr(obj), &hs);
   while (hp != 0) {
    /*
@@ -998,7 +1032,9 @@ done:
     vp->hPtr = 0;
     hp = Tcl_NextHashEntry(&hs);
   }
-  Tcl_DeleteHashTable(ObjVarTablePtr(obj));
+#endif
+  if (obj->variables.varTablePtr != NULL) 
+  	Tcl_DeleteHashTable(ObjVarTablePtr(obj));
   hp2 = obj->procs ? Tcl_FirstHashEntry(obj->procs, &hs2) : 0; 
   for (; hp2 != 0; hp2 = Tcl_NextHashEntry(&hs2)) {
     Tcl_CmdInfo* co = (Tcl_CmdInfo*)Tcl_GetHashValue(hp2);
@@ -1030,15 +1066,16 @@ PrimitiveOCreate(Tcl_Interp* in, CONST84 char* name, OTclClass* cl) {
 				PrimitiveODestroy);
   }
 #else
-  obj->variables.varTablePtr = (Tcl_HashTable *)ckalloc(sizeof(Tcl_HashTable));
-  if (obj != 0)
-    if (obj->variables.varTablePtr != 0) {
+  Tcl_VarHashInitialize(obj->variables.varTablePtr);
+  if (obj != 0) {
+    if (Tcl_IsVarHashInitialized(obj->variables.varTablePtr)) {
       PrimitiveOInit(obj, in, name, cl);
-      obj->id = Tcl_CreateCommand(in, name, OTclDispatch, (ClientData)obj,
-				PrimitiveODestroy);
-    } else {
+      obj->id = Tcl_CreateCommand(in, name, (Tcl_CmdProc *) OTclDispatch, 
+		    		  (ClientData)obj, PrimitiveODestroy);
+      } else {
       ckfree((char *)obj);
       obj = NULL;
+      }
     }
 #endif
   return obj;
@@ -1089,7 +1126,7 @@ PrimitiveCDestroy(ClientData cd) {
      * but under 7.4p1 it is too late, so check with info
      */
 
-    args[0] = Tcl_GetCommandName(in, obj->id);
+    args[0] = (char *) Tcl_GetCommandName(in, obj->id);
     if (Tcl_GetCommandInfo(in, args[0], &info))
       (void) OTclDispatch(cd, in, 2, args);
     obj->id = 0;
@@ -1108,7 +1145,7 @@ PrimitiveCDestroy(ClientData cd) {
     for (;;) {
       inst = (OTclObject*)Tcl_GetHashKey(&cl->instances, hPtr);
       if (inst != (OTclObject*)cl) {
-        CONST84 char* name = Tcl_GetCommandName(inst->teardown, inst->id);
+        CONST84 char* name = (char *) Tcl_GetCommandName(inst->teardown, inst->id);
 	(void)Tcl_DeleteCommand(inst->teardown, name);
 	break;
       }
@@ -1169,18 +1206,19 @@ PrimitiveCCreate(Tcl_Interp* in, CONST84 char* name, OTclClass* class){
 				PrimitiveCDestroy);
   }
 #else
-  cl->object.variables.varTablePtr = (Tcl_HashTable *)ckalloc(sizeof(Tcl_HashTable));
-  if (cl != 0) 
-    if (cl->object.variables.varTablePtr != 0) {
-      OTclObject* obj = (OTclObject*)cl;
+  Tcl_VarHashInitialize(cl->object.variables.varTablePtr);
+  if (cl != 0) {
+    if (Tcl_IsVarHashInitialized(cl->object.variables.varTablePtr)) {
+      OTclObject* obj = &cl->object;
       PrimitiveOInit(obj, in, name, class);
       PrimitiveCInit(cl, in, name, class);
-      obj->id = Tcl_CreateCommand(in, name, OTclDispatch, (ClientData)cl,
-				PrimitiveCDestroy);
-    } else {
+      obj->id = Tcl_CreateCommand(in, name, (Tcl_CmdProc *) OTclDispatch, 
+				  (ClientData)cl, PrimitiveCDestroy);
+      } else {
       ckfree((char *)cl);
       cl = NULL;
     }
+  }
 #endif
   return cl;
 }
@@ -1255,17 +1293,18 @@ OTclODestroyMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[])
    * to deal with trace error messages
    */
 
+  if (obj->variables.varTablePtr == NULL) goto done;
   hp = Tcl_FirstHashEntry(ObjVarTablePtr(obj), &hs);
   while (hp != 0) {
     for (;;) {
-      Var* vp = (Var*)Tcl_GetHashValue(hp);
+      Var* vp = Tcl_VarHashGetValue(hp);
       if (!TclIsVarUndefined(vp)) break;
       hp = Tcl_NextHashEntry(&hs);
       if (hp == 0)
         goto done;
     }
     if (hp != 0) {
-      CONST84 char* name = Tcl_GetHashKey(ObjVarTablePtr(obj), hp);
+	  CONST84 char* name = Tcl_VarHashGetKey(ObjVarTablePtr(obj), hp);
       result = OTclUnsetInstVar(obj, in, name, TCL_LEAVE_ERR_MSG);
       if (result != TCL_OK) break;
     }
@@ -1279,7 +1318,7 @@ done:
 
   oid = obj->id; obj->id = 0;
   if (obj->teardown != 0) {
-    CONST84 char* name = Tcl_GetCommandName(in, oid);
+    CONST84 char* name = (char *) Tcl_GetCommandName(in, oid);
     return (Tcl_DeleteCommand(in, name) == 0) ? TCL_OK : TCL_ERROR;
   }
   
@@ -1332,7 +1371,7 @@ OTclOInfoMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
     }
   } else if (!strcmp(argv[4], "commands")) {
     if (argc > 6) return OTclErrArgCnt(in,argv[0],"info commands ?pat?");
-    ListKeys(in, obj->procs, (argc == 6) ? argv[5] : 0);
+    ListKeys(in, obj->procs, (argc == 6) ? argv[5] : 0, 0);
   } else if (!strcmp(argv[4], "procs")) {
     if (argc > 6) return OTclErrArgCnt(in,argv[0],"info procs ?pat?");
     ListProcKeys(in, obj->procs, (argc == 6) ? argv[5] : 0);
@@ -1348,7 +1387,7 @@ OTclOInfoMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
     return ListProcBody(in, obj->procs, argv[5]);
   } else if (!strcmp(argv[4], "vars")) {
     if (argc > 6) return OTclErrArgCnt(in,argv[0],"info vars ?pat?");
-    ListKeys(in, ObjVarTablePtr(obj), (argc == 6) ? argv[5] : 0);
+    ListKeys(in, ObjVarTablePtr(obj), (argc == 6) ? argv[5] : 0, 1);
   } else {
     return OTclErrBadVal(in, "an info option", argv[4]);
   }
@@ -1379,7 +1418,11 @@ OTclOProcMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
   }
   (void)RemoveMethod(obj->procs, argv[4], (ClientData)obj);
   if (op == 1) AddMethod(obj->procs, argv[4], proc.proc,
-			 proc.clientData, proc.deleteProc, proc.deleteData);
+#ifdef TCL_VERSION_8_5_OR_HIGHER
+			 proc.clientData, proc.objClientData, proc.deleteProc, proc.deleteData);
+#else
+			 proc.clientData, NULL, proc.deleteProc, proc.deleteData);
+#endif
   
   return (op != 0) ? TCL_OK : TCL_ERROR;
 }
@@ -1388,8 +1431,8 @@ OTclOProcMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
 static int
 OTclONextMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
   OTclObject* obj = OTclAsObject(in, cd);
-  CONST84 char* class = Tcl_GetVar(in, "class",0);
-  CONST84 char* method = Tcl_GetVar(in, "proc",0);
+  CONST84 char* class = (char *) Tcl_GetVar(in, "class",0);
+  CONST84 char* method = (char *) Tcl_GetVar(in, "proc",0);
 
   if (!obj) return OTclErrType(in, argv[0], "Object");
   if (argc < 4) return OTclErrArgCnt(in, argv[0], "next ?args?");
@@ -1478,7 +1521,7 @@ OTclOInstVarMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[])
   for (i=4; i<argc; i++) {
     int ac;
     CONST84 char **av;
-    result = Tcl_SplitList(in, argv[i], &ac, &av);
+    result = Tcl_SplitList(in, argv[i], &ac, (const char ***) &av);
     if (result != TCL_OK) break;
     if (ac == 1) {
       result = OTclOInstVarOne(obj, in, "1", av[0], av[0], 0);
@@ -1543,12 +1586,12 @@ OTclCCreateMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[])
   if (!pl) return OTclErrMsg(in, "no reachable alloc", TCL_STATIC);
   
   for (i=0; i<4; i++) args[i] = argv[i];
-  argv[0] = Tcl_GetCommandName(in, pl->cl->object.id);
+  argv[0] = (char *) Tcl_GetCommandName(in, pl->cl->object.id);
   argv[1] = argv[0];
   argv[2] = "";
   argv[3] = "alloc";
   cp = (cp != 0) ? cp : (ClientData)pl->cl;
-  result = (*proc)(cp, in, argc, argv);
+  result = (*proc)(cp, in, argc, (const char **) argv);
   for (i=0; i<4; i++) argv[i] = args[i];
   if (result != TCL_OK) return result;
 
@@ -1577,7 +1620,8 @@ OTclCSuperClassMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv
 
   if (!cl) return OTclErrType(in, argv[0], "Class");
   if (argc != 5) return OTclErrArgCnt(in, argv[0], "superclass <classes>");
-  if (Tcl_SplitList(in, argv[4], &ac, &av) != TCL_OK) return TCL_ERROR;
+  if (Tcl_SplitList(in, argv[4], &ac, (const char ***) &av) != TCL_OK)
+    return TCL_ERROR;
 
   scl = (OTclClass**)ckalloc(ac*sizeof(OTclClass*));
   for (i = 0; i < ac; i++) {
@@ -1750,7 +1794,7 @@ OTclCInfoMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
     if (pl) pl = pl->next;
     Tcl_ResetResult(in);
     for (; pl != 0; pl = pl->next) {
-      CONST84 char* name = Tcl_GetCommandName(in, pl->cl->object.id);
+      CONST84 char* name = (char *) Tcl_GetCommandName(in, pl->cl->object.id);
       if (pattern && !Tcl_StringMatch(name, pattern)) continue;
       Tcl_AppendElement(in, name);
     }
@@ -1759,7 +1803,7 @@ OTclCInfoMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
     ListInstanceKeys(in, &cl->instances, (argc == 6) ? argv[5] : 0);
   } else if (!strcmp(argv[4], "instcommands")) {
     if (argc > 6) return OTclErrArgCnt(in,argv[0],"info instcommands ?pat?");
-    ListKeys(in, &cl->instprocs, (argc == 6) ? argv[5] : 0);
+    ListKeys(in, &cl->instprocs, (argc == 6) ? argv[5] : 0, 0);
   } else if (!strcmp(argv[4], "instprocs")) {
     if (argc > 6) return OTclErrArgCnt(in,argv[0],"info instprocs ?pat?");
     ListProcKeys(in, &cl->instprocs, (argc == 6) ? argv[5] : 0);
@@ -1800,7 +1844,11 @@ OTclCInstProcMethod(ClientData cd, Tcl_Interp* in, int argc, CONST84 char*argv[]
 
   (void)RemoveMethod(&cl->instprocs, argv[4], (ClientData)cl);
   if (op == 1) AddMethod(&cl->instprocs, argv[4], proc.proc,
-			 proc.clientData, proc.deleteProc, proc.deleteData);
+#ifdef TCL_VERSION_8_5_OR_HIGHER
+			 proc.clientData, proc.objClientData, proc.deleteProc, proc.deleteData);
+#else
+			 proc.clientData, NULL, proc.deleteProc, proc.deleteData);
+#endif			
   
   return (op != 0) ? TCL_OK : TCL_ERROR;
 }
@@ -1829,7 +1877,7 @@ OTclGetObject(Tcl_Interp* in, CONST84 char* name) {
   Tcl_CmdInfo info;
   OTclObject* obj = 0;
   if (Tcl_GetCommandInfo(in, name, &info))
-    if (info.proc == OTclDispatch)
+    if (info.proc == (Tcl_CmdProc *) OTclDispatch)
       obj = OTclAsObject(in, info.clientData);
   return obj;
 }
@@ -1840,7 +1888,7 @@ OTclGetClass(Tcl_Interp* in, CONST84 char* name) {
   Tcl_CmdInfo info;
   OTclClass* cl = 0;
   if (Tcl_GetCommandInfo(in, name, &info))
-    if (info.proc == OTclDispatch)
+    if (info.proc == (Tcl_CmdProc *) OTclDispatch)
       cl = OTclAsClass(in, info.clientData);
   return cl;
 }
@@ -1849,7 +1897,7 @@ OTclGetClass(Tcl_Interp* in, CONST84 char* name) {
 extern OTclObject*
 OTclCreateObject(Tcl_Interp* in, CONST84 char* name, OTclClass* cl) {
   CONST84 char* args[3];
-  args[0] = Tcl_GetCommandName(in, cl->object.id);
+  args[0] = (char *) Tcl_GetCommandName(in, cl->object.id);
   args[1] = "create";
   args[2] = name;
   if (OTclDispatch((ClientData)cl,in,3,args) != TCL_OK) return 0;
@@ -1860,7 +1908,7 @@ OTclCreateObject(Tcl_Interp* in, CONST84 char* name, OTclClass* cl) {
 extern OTclClass*
 OTclCreateClass(Tcl_Interp* in, CONST84 char* name, OTclClass* cl){
   CONST84 char* args[3];
-  args[0] = Tcl_GetCommandName(in, cl->object.id);
+  args[0] = (char *) Tcl_GetCommandName(in, cl->object.id);
   args[1] = "create";
   args[2] = name;
   if (OTclDispatch((ClientData)cl,in,3,args) != TCL_OK) return 0;
@@ -1871,7 +1919,7 @@ OTclCreateClass(Tcl_Interp* in, CONST84 char* name, OTclClass* cl){
 extern int
 OTclDeleteObject(Tcl_Interp* in, OTclObject* obj) {
   CONST84 char* args[2];
-  args[0] = Tcl_GetCommandName(in, obj->id);
+  args[0] = (char *) Tcl_GetCommandName(in, obj->id);
   args[1] = "destroy";
   return OTclDispatch((ClientData)obj, in, 2, args);
 }
@@ -1880,7 +1928,7 @@ OTclDeleteObject(Tcl_Interp* in, OTclObject* obj) {
 extern int
 OTclDeleteClass(Tcl_Interp* in, OTclClass* cl) {
   CONST84 char* args[2];
-  args[0] = Tcl_GetCommandName(in, cl->object.id);
+  args[0] = (char *) Tcl_GetCommandName(in, cl->object.id);
   args[1] = "destroy";
   return OTclDispatch((ClientData)cl, in, 2, args);
 }
@@ -1896,7 +1944,7 @@ OTclAddPMethod(OTclObject* obj, char* nm, Tcl_CmdProc* proc,
     obj->procs = (Tcl_HashTable*)ckalloc(sizeof(Tcl_HashTable));
     Tcl_InitHashTable(obj->procs, TCL_STRING_KEYS);
   }
-  AddMethod(obj->procs, nm, proc, cd, dp, cd);
+  AddMethod(obj->procs, nm, proc, cd, NULL, dp, cd);
 }
 
 
@@ -1905,7 +1953,7 @@ OTclAddIMethod(OTclClass* cl, char* nm, Tcl_CmdProc* proc,
 	       ClientData cd, Tcl_CmdDeleteProc* dp)
 {
   (void)RemoveMethod(&cl->instprocs, nm, (ClientData)cl);
-  AddMethod(&cl->instprocs, nm, proc, cd, dp, cd);
+  AddMethod(&cl->instprocs, nm, proc, cd, NULL, dp, cd);
 }
 
 
@@ -1956,8 +2004,8 @@ OTclNextMethod(OTclObject* obj, Tcl_Interp* in, int argc, CONST84 char*argv[]) {
   ncl = SearchCMethod(pl, argv[3], &proc, &cp, 0);  
   if (proc != 0) {
     cp = (cp != 0) ? cp : (ClientData)obj;
-    argv[2] = Tcl_GetCommandName(in, ncl->object.id);
-    result = (*proc)(cp, in, argc, argv);
+    argv[2] = (char *) Tcl_GetCommandName(in, ncl->object.id);
+    result = (*proc)(cp, in, argc, (const char **) argv);
     argv[2] = class;
   }
   return result;
@@ -1972,7 +2020,7 @@ OTclSetInstVar(OTclObject* obj,Tcl_Interp* in,
   CONST84 char* result;
 
   iPtr->varFramePtr = &obj->variables;
-  result = Tcl_SetVar(in, name, value, flgs);
+  result = (char *) Tcl_SetVar(in, name, value, flgs);
   iPtr->varFramePtr = saved;
   return result;
 }
@@ -1985,7 +2033,7 @@ OTclGetInstVar(OTclObject* obj, Tcl_Interp* in, CONST84 char* name, int flgs){
   CONST84 char* result;
 
   iPtr->varFramePtr = &obj->variables;
-  result = Tcl_GetVar(in, name, flgs);
+  result = (char *) Tcl_GetVar(in, name, flgs);
   iPtr->varFramePtr = saved;
   return result;
 }
@@ -2157,22 +2205,22 @@ Otcl_Init(Tcl_Interp* in) {
    * and fill them with functionality
    */
   
-  OTclAddPMethod((OTclObject*)theobj, "alloc", OTclOAllocMethod, 0, 0);
-  OTclAddIMethod(theobj, "init", OTclOInitMethod, 0, 0);
-  OTclAddIMethod(theobj, "destroy", OTclODestroyMethod, 0, 0);
-  OTclAddIMethod(theobj, "class", OTclOClassMethod, 0, 0);
-  OTclAddIMethod(theobj, "info", OTclOInfoMethod, 0, 0);
-  OTclAddIMethod(theobj, "proc", OTclOProcMethod, 0, 0);
-  OTclAddIMethod(theobj, "next", OTclONextMethod, 0, 0);
-  OTclAddIMethod(theobj, "set", OTclOSetMethod, 0, 0);
-  OTclAddIMethod(theobj, "unset", OTclOUnsetMethod, 0, 0);
-  OTclAddIMethod(theobj, "instvar", OTclOInstVarMethod, 0, 0);
+  OTclAddPMethod((OTclObject*)theobj, "alloc", (Tcl_CmdProc *) OTclOAllocMethod, 0, 0);
+  OTclAddIMethod(theobj, "init", (Tcl_CmdProc *) OTclOInitMethod, 0, 0);
+  OTclAddIMethod(theobj, "destroy", (Tcl_CmdProc *) OTclODestroyMethod, 0, 0);
+  OTclAddIMethod(theobj, "class", (Tcl_CmdProc *) OTclOClassMethod, 0, 0);
+  OTclAddIMethod(theobj, "info", (Tcl_CmdProc *) OTclOInfoMethod, 0, 0);
+  OTclAddIMethod(theobj, "proc", (Tcl_CmdProc *) OTclOProcMethod, 0, 0);
+  OTclAddIMethod(theobj, "next", (Tcl_CmdProc *) OTclONextMethod, 0, 0);
+  OTclAddIMethod(theobj, "set", (Tcl_CmdProc *) OTclOSetMethod, 0, 0);
+  OTclAddIMethod(theobj, "unset", (Tcl_CmdProc *) OTclOUnsetMethod, 0, 0);
+  OTclAddIMethod(theobj, "instvar", (Tcl_CmdProc *) OTclOInstVarMethod, 0, 0);
 
-  OTclAddPMethod((OTclObject*)thecls, "alloc", OTclCAllocMethod, 0, 0);
-  OTclAddIMethod(thecls, "create", OTclCCreateMethod, 0, 0);
-  OTclAddIMethod(thecls, "superclass", OTclCSuperClassMethod, 0, 0);
-  OTclAddIMethod(thecls, "info", OTclCInfoMethod, 0, 0);
-  OTclAddIMethod(thecls, "instproc", OTclCInstProcMethod, 0, 0);
+  OTclAddPMethod((OTclObject*)thecls, "alloc", (Tcl_CmdProc *) OTclCAllocMethod, 0, 0);
+  OTclAddIMethod(thecls, "create", (Tcl_CmdProc *) OTclCCreateMethod, 0, 0);
+  OTclAddIMethod(thecls, "superclass", (Tcl_CmdProc *) OTclCSuperClassMethod, 0, 0);
+  OTclAddIMethod(thecls, "info", (Tcl_CmdProc *) OTclCInfoMethod, 0, 0);
+  OTclAddIMethod(thecls, "instproc", (Tcl_CmdProc *) OTclCInstProcMethod, 0, 0);
 
   /*
    * with some methods and library procs in tcl - they could go in a
